var Sequencing = function()
{
	var sequencing = {};
	var debug = 0;
	
	var runParametersErrorTarget;
	var runParametersIsValid = false;
	var runParameters;
	var flowCell;

	// Page initialization
	sequencing.initPage = function()
	{
		var frm = document.forms['reggie'];
		// Step 1
		Events.addEventHandler('runParametersXml', 'change', sequencing.runParametersXmlFileOnChange);
		Events.addEventHandler('sequencer', 'change', sequencing.sequencerOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', sequencing.validateStep1);
		
		Buttons.addClickHandler('searchRunArchive', sequencing.searchRunArchive);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', sequencing.submit);

		Doc.show('step-1');
		Doc.show('goregister');
		
		Reggie.loadProtocols('SEQUENCING_PROTOCOL', 'sequencingProtocol');
		Reggie.loadHardware('SEQUENCER', 'sequencer', 'SERIAL_NUMBER');
		
		// Load open grid clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, sequencing.clustersLoaded);

		// Load flow cells
		url = '../SequencingRun.servlet?ID='+App.getSessionId() + '&cmd=GetFlowCellsToSequence';
		Doc.addClass('flowCells', 'list-loading');
		frm.flowCells[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, sequencing.flowCellsLoaded);
	}

	sequencing.validateStep1 = function(event)
	{
		if (!runParametersIsValid) event.preventDefault();
	}
	
	sequencing.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.clusters;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.id, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
		}
		if (frm.clusters.length == 0)
		{
			frm.clusters[frm.clusters.length] = new Option('No available Open Grid clusters');
			frm.flowCells.disabled = true;
			frm.clusters.disabled = true;
			Doc.addClass('searchRunArchive', 'disabled');
		}
	}

	sequencing.flowCellsLoaded = function(response)
	{
		Doc.removeClass('flowCells', 'list-loading');
		var frm = document.forms['reggie'];
		var flowCells = response.flowCells;
		frm.flowCells.length = 0;
		for (var i = 0; i < flowCells.length; i++)
		{
			var fc = flowCells[i];
			var option = sequencing.getOptionForFlowCell(fc);
			frm.flowCells[frm.flowCells.length] = option;
		}
		if (frm.flowCells.length == 0)
		{
			frm.flowCells[frm.flowCells.length] = new Option('No available flow cells');
			frm.flowCells.disabled = true;
			frm.clusters.disabled = true;
			Doc.addClass('searchRunArchive', 'disabled');
		}
	}
	
	sequencing.getOptionForFlowCell = function(flowCell)
	{
		var name = flowCell.name+': ';
		var numPools = flowCell.pools.length;
		var firstPoolNum = sequencing.getPoolNum(flowCell.pools[0].name);
		var lastPoolNum = sequencing.getPoolNum(flowCell.pools[flowCell.pools.length-1].name);

		if (lastPoolNum - firstPoolNum == numPools - 1)
		{
			if (numPools > 1)
			{
				// Display: PoolN -- PoolY
				name += flowCell.pools[0].name + ' — ' + flowCell.pools[flowCell.pools.length-1].name;
			}
			else
			{
				name += flowCell.pools[0].name;
			}
		}
		else
		{
			// Display: PoolN + x more...
			name += flowCell.pools[0].name + ' + ' + (numPools-1) + ' more...';
		}
		
		if (flowCell.FlowCellType)
		{
			name += ' [' + Strings.encodeTags(flowCell.FlowCellType) + ']';
		}

		// Tooltip is always all pools
		var title = '';
		for (var poolNo=0; poolNo < numPools; poolNo++)
		{
			var pool = flowCell.pools[poolNo];
			if (poolNo > 0) title += ', ';
			title += pool.name;
		}

		var option = new Option(name, flowCell.id);
		option.title = title;
		option.flowCell = flowCell;	
		return option;
	}

	sequencing.getPoolNum = function(poolName)
	{
		var num = poolName.match(/Pool(\d+)/);
		return num ? parseInt(num[1], 10) : null;
	}

	
	sequencing.clearRunParametersInfo = function()
	{
		// Clear information
		Doc.element('experimentName').innerHTML = '';
		Doc.element('externalId').innerHTML = '';
		Doc.element('runId').innerHTML = '';
		Doc.element('position').innerHTML = '';
		Doc.element('runNumber').innerHTML = '';
		Doc.element('startDate').innerHTML = '';
		Doc.element('sequencingCycles').innerHTML = '';
		Doc.element('flowCellName').innerHTML = 'Flow cell information';
		Doc.element('pools').innerHTML = '';
		Doc.element('libPlate').innerHTML = '';
		Doc.element('dataFolder').innerHTML = '';
		Doc.hide('existing-sequencing-runs');
		Doc.element('existing-sequencing-runs').innerHTML = '';
		
		runParametersIsValid = false;
		Wizard.setInputStatus('searchRunArchive');
		Wizard.setInputStatus('runParametersXml');
		Wizard.setInputStatus('sequencingCycles');
		Wizard.setInputStatus('externalId');
	}
	
	sequencing.searchRunArchive = function()
	{
		sequencing.clearRunParametersInfo();
		
		var frm = document.forms['reggie'];
		var flowCell = frm.flowCells[frm.flowCells.selectedIndex].flowCell;
		
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=SearchRunArchive';
		url += '&FlowCellID='+encodeURIComponent(flowCell.FlowCellID);
		url += '&cluster='+encodeURIComponent(frm.clusters.value);
		
		runParametersErrorTarget = 'searchRunArchive';
		Wizard.setInputStatus('searchRunArchive', 'checking', 'Searching...');
		Wizard.asyncJsonRequest(url, sequencing.runArchiveSearched);
	}
	
	sequencing.runArchiveSearched = function(response)
	{
		Wizard.setInputStatus('searchRunArchive');
		if (response.exitStatus != 0)
		{
			Wizard.setInputStatus('searchRunArchive', 'invalid', Strings.encodeTags(response.stderr).replace(/\n/g, '<br>'));
			return;
		}
		
		Doc.element('dataFolder').innerHTML = Strings.encodeTags(response.dataFolder);
		
		runParameters = sequencing.parseRunParametersXmlFile(response.runParameters);
		sequencing.validateAndCopyRunParameters(runParameters);
		var frm = document.forms['reggie'];
		frm.comments.focus();
	}
	
	sequencing.runParametersXmlFileOnChange = function(event)
	{
		sequencing.clearRunParametersInfo();
		
		var frm = document.forms['reggie'];
		var file = event.target.files[0];
		
		runParametersErrorTarget = 'runParametersXml';
		var reader = new FileReader();
		reader.onload = sequencing.runParametersXmlFileLoaded;
		reader.readAsText(file);
	}

	
	sequencing.runParametersXmlFileLoaded = function(event)
	{
		runParameters = sequencing.parseRunParametersXmlFile(event.target.result);
		sequencing.validateAndCopyRunParameters(runParameters);
		var frm = document.forms['reggie'];
		frm.comments.focus();
	}

	sequencing.parseRunParametersXmlFile = function(data)
	{
		var doc;
		var info = {};
		try
		{
			var parser = new DOMParser();
			doc = parser.parseFromString(data, "application/xml");
		
			if (doc.documentElement.tagName == 'parsererror')
			{
				throw new Error(doc.documentElement.firstChild.textContent.split(/\n/)[0]);
			}
			
			if (sequencing.isHiSeqRunParameters(doc))
			{
				info.ExperimentName = sequencing.getTextValue(doc, 'ExperimentName');
				info.RunID = sequencing.getTextValue(doc, 'RunID');
				info.HiSeqPosition = sequencing.getTextValue(doc, 'FCPosition');
				info.ScannerID = sequencing.getTextValue(doc, 'ScannerID');
				info.ScanNumber = parseInt(sequencing.getTextValue(doc, 'ScanNumber'));
				info.FlowCellID = sequencing.getTextValue(doc, 'Barcode');
				info.SequencingCycles = sequencing.getTextValue(doc, 'Read1') + '-' + sequencing.getTextValue(doc, 'IndexRead1') + '-' + sequencing.getTextValue(doc, 'Read2');
				info.SequencingStart = '20'+sequencing.getTextValue(doc, 'RunStartDate');
			}
			else
			{
				info.ExperimentName = sequencing.getTextValue(doc, 'ExperimentName');
				info.RunID = sequencing.getTextValue(doc, 'RunID');
				info.HiSeqPosition = 'A';
				info.ScannerID = sequencing.getTextValue(doc, 'InstrumentID');
				info.ScanNumber = parseInt(sequencing.getTextValue(doc, 'RunNumber'));
				info.FlowCellID = sequencing.getTextValue(doc, 'FlowCellSerial');
				info.SequencingCycles = sequencing.getTextValue(doc, 'Read1') + '-' + sequencing.getTextValue(doc, 'Index1Read') + '-' + sequencing.getTextValue(doc, 'Read2');
				info.SequencingStart = '20'+sequencing.getTextValue(doc, 'RunStartDate');
			}			
		}
		catch (err)
		{
			Wizard.setInputStatus(runParametersErrorTarget, 'invalid', err.toString());
			info.error = true;
		}
		
		if (debug) App.debug(JSON.stringify(info));
		return info;
	}

	sequencing.isHiSeqRunParameters = function(doc)
	{
		var el = doc.querySelector('ApplicationName');
		return el && el.textContent.indexOf('HiSeq') >= 0;
	}

	sequencing.getTextValue = function(doc, selector)
	{
		var el = doc.querySelector(selector);
		if (el == null)
		{
			throw new Error("Can't find tag &lt;"+selector+"&gt;");
		}
		return el.textContent;
	}

	
	sequencing.validateAndCopyRunParameters = function(info)
	{
		var frm = document.forms['reggie'];

		Doc.element('experimentName').innerHTML = info.ExperimentName || '';
		Doc.element('externalId').innerHTML = info.FlowCellID || '';
		Doc.element('runId').innerHTML = info.RunID || '';
		Doc.element('position').innerHTML = info.HiSeqPosition || '';
		Doc.element('runNumber').innerHTML = info.ScanNumber || '';
		Doc.element('startDate').innerHTML = Reggie.reformatDate(info.SequencingStart) || '';
		Doc.element('sequencingCycles').innerHTML = info.SequencingCycles || '';

		if (info.ScannerID)
		{
			for (var seqNo = 0; seqNo < frm.sequencer.length; seqNo++)
			{
				var sequencer = frm.sequencer[seqNo].item;
				if (sequencer && sequencer.SerialNumber == info.ScannerID)
				{
					frm.sequencer.selectedIndex = seqNo;
					seqNo = frm.sequencer.length;
				}
			}
			sequencing.sequencerOnChange();
		}
		if (info.error) return;
		
		var url = '../FlowCell.servlet?ID='+App.getSessionId();
		url += '&cmd=GetFlowCellByBarcode';
		url += '&barcode='+encodeURIComponent(info.FlowCellID);

		Wizard.showLoadingAnimation('Loading flow cell information...');
		Wizard.asyncJsonRequest(url, sequencing.flowCellInfoLoaded);
	}
		
	sequencing.flowCellInfoLoaded = function(response)
	{
		flowCell = response.flowCell;
		
		if (flowCell == null)
		{
			Doc.element('flowCellName').innerHTML = 'Can\'t find flow cell ' + Strings.encodeTags(runParameters.FlowCellID);
			Wizard.setInputStatus(runParametersErrorTarget, 'invalid', 'Flow cell with ID='+runParameters.FlowCellID+' not found.');
			return;
		}
		
		Doc.element('flowCellName').innerHTML = Strings.encodeTags(flowCell.name);
		if (flowCell.externalId != runParameters.FlowCellID)
		{
			Wizard.setInputStatus('externalId', 'warning', 'Not same as registered barcode ('+Strings.encodeTags(flowCell.externalId)+').');
		}
		if (flowCell.SequencingCycles != runParameters.SequencingCycles)
		{
			Wizard.setInputStatus('sequencingCycles', 'warning', 'Not same as planned SequencingCycles ('+Strings.encodeTags(flowCell.SequencingCycles)+').');
		}

		var pools = flowCell.pools;
		var html = '';
		var libPlateHtml = '';
		for (var poolNo = 0; poolNo < pools.length; poolNo++)
		{
			var pool = pools[poolNo];
			if (poolNo > 0) html += ', ';
			html += Strings.encodeTags(pool.name);
			if (pool.LibPlate && libPlateHtml.indexOf(pool.LibPlate.name) == -1)
			{
				if (libPlateHtml.length > 0) libPlateHtml += ', ';
				libPlateHtml += Strings.encodeTags(pool.LibPlate.name);
			}
		}
		Doc.element('pools').innerHTML = html;
		Doc.element('libPlate').innerHTML = libPlateHtml || 'n/a';

		
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=GetSequencingRunsForFlowCell';
		url += '&flowCell='+flowCell.id;
		Wizard.showLoadingAnimation('Loading sequencing runs information...');
		Wizard.asyncJsonRequest(url, sequencing.existingSequencingRunsLoaded);
	}
	
	sequencing.existingSequencingRunsLoaded = function(response)
	{
		
		var seqRuns = response.sequencingRuns;
		
		var html = '';
		html += '<tr class="section-header">';
		html += '<td colspan="3">Existing sequencing runs</td>';
		html += '<td class="help">Start and end date is shown for other registered sequencing runs.</td>';
		html += '</tr>';

		var runExists = null;
		if (seqRuns.length > 0)
		{
			for (var seqNo = 0; seqNo < seqRuns.length; seqNo++)
			{
				var seqRun = seqRuns[seqNo];
				html += '<tr>';
				html += '<td class="prompt">'+seqRun.name+'</td>';
				html += '<td class="info" colspan="2">';
				html += Reggie.reformatDate(seqRun.SequencingStart, '(no start date)') + ' -- ' + Reggie.reformatDate(seqRun.SequencingEnd, '(no end date)');
				if (seqRun.SequencingResult)
				{
					html += ': ' + seqRun.SequencingResult;
				}
				html += '</td>';
				html += '<td class="help"></td>';
				html += '</tr>';
				if (seqRun.DataFilesFolder == runParameters.RunID)
				{
					runExists = seqRun;
				}
			}
		}
		else
		{
			html += '<tr >';
			html += '<td class="prompt"></td>';
			html += '<td class="info" colspan="2">';
			html += 'none';
			html += '</td>';
			html += '<td class="help"></td>';
			html += '</tr>';
		}
		
		Doc.show('existing-sequencing-runs');
		Doc.element('existing-sequencing-runs').innerHTML = html;
		if (runExists != null)
		{
			Wizard.setInputStatus(runParametersErrorTarget, 'invalid', 'This sequencing run has already been registered: ' + Strings.encodeTags(runExists.name));
			return;
		}
		
		Wizard.setInputStatus(runParametersErrorTarget, 'valid');
		runParametersIsValid = true;
		
	}

	sequencing.sequencerOnChange = function()
	{
		var frm = document.forms['reggie'];
		var sequencer = frm.sequencer[frm.sequencer.selectedIndex].item;
		
		var scannerID = runParameters ? runParameters.ScannerID : null;
		if (scannerID && sequencer)
		{
			if (scannerID != sequencer.SerialNumber)
			{
				Wizard.setInputStatus('sequencer', 'warning', 'Serial number mismatch: ' + Strings.encodeTags(scannerID));
				return;
			}
		}
		Wizard.setInputStatus('sequencer', 'valid');
	}

	sequencing.submit = function()
	{
		var frm = document.forms['reggie'];
		
		var submitInfo = {};
		submitInfo.flowCell = flowCell;
		submitInfo.runParameters = runParameters;
		
		var sequencingRun = {};
		sequencingRun.operator = frm.sequencingOperator.value;
		sequencingRun.sequencer = parseInt(frm.sequencer.value);
		sequencingRun.protocol = parseInt(frm.sequencingProtocol.value);
		sequencingRun.comments = frm.comments.value;
		submitInfo.sequencingRun = sequencingRun;
		
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterSequencingStarted';
		
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, sequencing.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	sequencing.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return sequencing;
}();

Doc.onLoad(Sequencing.initPage);

